"""
Type annotations for botocore.response module.

Copyright 2025 Vlad Emelianov
"""

import logging
from collections.abc import Iterator
from io import IOBase
from typing import IO, Any

import requests
from botocore.model import OperationModel

logger: logging.Logger

class StreamingBody(IOBase):
    def __init__(self, raw_stream: IO[bytes], content_length: int) -> None: ...
    def set_socket_timeout(self, timeout: float) -> None: ...
    def readable(self) -> bool: ...
    def read(self, amt: int | None = ...) -> bytes: ...
    def readinto(self, b: IO[bytes]) -> int: ...
    # FIXME: Signature of "readlines" incompatible with supertype "IOBase"
    def readlines(self) -> list[bytes]: ...  # type: ignore [override]
    def __iter__(self) -> Iterator[bytes]: ...
    def iter_lines(self, chunk_size: int = ..., keepends: bool = ...) -> Iterator[bytes]: ...
    def iter_chunks(self, chunk_size: int = ...) -> Iterator[bytes]: ...
    def tell(self) -> int: ...
    def close(self) -> None: ...
    def next(self) -> bytes: ...

def get_response(
    operation_model: OperationModel, http_response: requests.Response
) -> tuple[requests.Response, Any]: ...



"""
Type annotations for botocore.docs.shape module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping
from typing import Any

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.hooks import BaseEventHooks
from botocore.model import Shape

class ShapeDocumenter:
    EVENT_NAME: str = ...
    def __init__(
        self,
        service_name: str,
        operation_name: str,
        event_emitter: BaseEventHooks,
        context: Mapping[str, Any] | None = ...,
    ) -> None: ...
    def traverse_and_document_shape(
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        include: Any = ...,
        exclude: Any = ...,
        name: str | None = ...,
        is_required: bool = ...,
    ) -> None: ...



"""
Type annotations for botocore.configprovider module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping, Sequence
from logging import Logger
from typing import Any, Callable, TypeVar

from botocore.session import Session
from botocore.utils import IMDSRegionProvider

BOTOCORE_DEFAUT_SESSION_VARIABLES: dict[str, tuple[Any, ...]]
DEFAULT_S3_CONFIG_VARS: dict[str, tuple[Any, ...]]
DEFAULT_PROXIES_CONFIG_VARS: dict[str, tuple[Any, ...]]

logger: Logger = ...

_R = TypeVar("_R")

def create_botocore_default_config_mapping(session: Any) -> Any: ...

class DefaultConfigResolver:
    def __init__(self, default_config_data: Mapping[str, Any]) -> None: ...
    def get_default_modes(self) -> list[str]: ...
    def get_default_config_values(self, mode: str) -> Any: ...

class ConfigChainFactory:
    def __init__(self, session: Any, environ: Any | None = ...) -> None: ...
    def create_config_chain(
        self,
        instance_name: str | None = ...,
        env_var_names: str | list[str] | None = ...,
        config_property_names: str | Sequence[str] | None = ...,
        default: Any | None = ...,
        conversion_func: Callable[..., Any] | None = ...,
    ) -> ChainProvider: ...

class ConfigValueStore:
    def __init__(self, mapping: Mapping[str, Any] | None = ...) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def __copy__(self: _R) -> _R: ...
    def get_config_variable(self, logical_name: str) -> Any: ...
    def get_config_provider(self, logical_name: str) -> BaseProvider: ...
    def set_config_variable(self, logical_name: str, value: Any) -> None: ...
    def clear_config_variable(self, logical_name: str) -> None: ...
    def set_config_provider(self, logical_name: str, provider: Any) -> None: ...

class SmartDefaultsConfigStoreFactory:
    def __init__(
        self,
        default_config_resolver: DefaultConfigResolver,
        imds_region_provider: IMDSRegionProvider,
    ) -> None: ...
    def merge_smart_defaults(
        self, config_store: ConfigValueStore, mode: str, region_name: str
    ) -> None: ...
    def resolve_auto_mode(self, region_name: str) -> str: ...

class BaseProvider:
    def provide(self) -> Any: ...

class ChainProvider(BaseProvider):
    def __init__(
        self,
        providers: Sequence[BaseProvider] | None = ...,
        conversion_func: Callable[..., Any] | None = ...,
    ) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def provide(self) -> Any: ...
    def set_default_provider(self, default_provider: BaseProvider) -> None: ...

class InstanceVarProvider(BaseProvider):
    def __init__(self, instance_var: str, session: Session) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def provide(self) -> Any: ...

class ScopedConfigProvider(BaseProvider):
    def __init__(self, config_var_name: str, session: Session) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def provide(self) -> Any: ...

class EnvironmentProvider(BaseProvider):
    def __init__(self, name: str, env: Mapping[str, str]) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def provide(self) -> Any: ...

class SectionConfigProvider(BaseProvider):
    def __init__(
        self,
        section_name: str,
        session: Session,
        override_providers: dict[str, BaseProvider] | None = ...,
    ) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def provide(self) -> Any: ...
    def set_default_provider(self, key: str, default_provider: BaseProvider) -> None: ...

class ConstantProvider(BaseProvider):
    def __init__(self, value: Any) -> None: ...
    def __deepcopy__(self: _R, memo: dict[str, Any]) -> _R: ...
    def provide(self) -> Any: ...

class ConfiguredEndpointProvider(BaseProvider):
    def __init__(
        self,
        full_config: Mapping[str, Any],
        scoped_config: Mapping[str, Any],
        client_name: str,
        environ: Mapping[str, str] | None = ...,
    ) -> None: ...
    def provide(self) -> Any: ...



"""
Type annotations for botocore.config module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping
from typing import Any, Literal, TypedDict, TypeVar

from botocore.compat import OrderedDict as OrderedDict
from botocore.endpoint import DEFAULT_TIMEOUT as DEFAULT_TIMEOUT
from botocore.endpoint import MAX_POOL_CONNECTIONS as MAX_POOL_CONNECTIONS
from botocore.exceptions import InvalidMaxRetryAttemptsError as InvalidMaxRetryAttemptsError
from botocore.exceptions import InvalidRetryConfigurationError as InvalidRetryConfigurationError
from botocore.exceptions import InvalidRetryModeError as InvalidRetryModeError
from botocore.exceptions import InvalidS3AddressingStyleError as InvalidS3AddressingStyleError

class _RetryDict(TypedDict, total=False):
    total_max_attempts: int
    max_attempts: int
    mode: Literal["legacy", "standard", "adaptive"]

class _S3Dict(TypedDict, total=False):
    use_accelerate_endpoint: bool
    payload_signing_enabled: bool
    addressing_style: Literal["auto", "virtual", "path"]
    us_east_1_regional_endpoint: Literal["regional", "legacy"]

class _ProxiesConfigDict(TypedDict, total=False):
    proxy_ca_bundle: str
    proxy_client_cert: str | tuple[str, str]
    proxy_use_forwarding_for_https: bool

_Config = TypeVar("_Config", bound=Config)

class Config:
    OPTION_DEFAULTS: OrderedDict[str, None]
    NON_LEGACY_OPTION_DEFAULTS: dict[str, None]
    def __init__(
        self,
        region_name: str | None = None,
        signature_version: str | None = None,
        user_agent: str | None = None,
        user_agent_extra: str | None = None,
        connect_timeout: float | None = 60,
        read_timeout: float | None = 60,
        parameter_validation: bool | None = True,
        max_pool_connections: int | None = 10,
        proxies: Mapping[str, str] | None = None,
        proxies_config: _ProxiesConfigDict | None = None,
        s3: _S3Dict | None = None,
        retries: _RetryDict | None = None,
        client_cert: str | tuple[str, str] | None = None,
        inject_host_prefix: bool | None = True,
        endpoint_discovery_enabled: bool | None = None,
        use_dualstack_endpoint: bool | None = None,
        use_fips_endpoint: bool | None = None,
        defaults_mode: bool | None = None,
        tcp_keepalive: bool | None = False,
        request_min_compression_size_bytes: int | None = None,
        disable_request_compression: bool | None = None,
        sigv4a_signing_region_set: str | None = None,
        client_context_params: Mapping[str, Any] | None = None,
        request_checksum_calculation: Literal["when_supported", "when_required"] | None = None,
        response_checksum_validation: Literal["when_supported", "when_required"] | None = None,
        account_id_endpoint_mode: Literal["preferred", "disabled", "required"] | None = None,
    ) -> None: ...
    def merge(self: _Config, other_config: _Config) -> _Config: ...
    @property
    def inject_host_prefix(self) -> bool: ...
    @inject_host_prefix.setter
    def inject_host_prefix(self, value: bool) -> None: ...



"""
Type annotations for botocore.docs.method module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Sequence
from typing import Any

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.hooks import BaseEventHooks
from botocore.model import OperationModel

AWS_DOC_BASE: str = ...

def get_instance_public_methods(instance: Any) -> dict[str, Any]: ...
def document_model_driven_signature(
    section: DocumentStructure,
    name: str,
    operation_model: OperationModel,
    include: Sequence[str] | None = ...,
    exclude: Sequence[str] | None = ...,
) -> None: ...
def document_custom_signature(
    section: DocumentStructure,
    name: str,
    method: Any,
    include: Sequence[str] | None = ...,
    exclude: Sequence[str] | None = ...,
) -> None: ...
def document_custom_method(section: DocumentStructure, method_name: str, method: Any) -> None: ...
def document_model_driven_method(
    section: DocumentStructure,
    method_name: str,
    operation_model: OperationModel,
    event_emitter: BaseEventHooks,
    method_description: str | None = ...,
    example_prefix: str | None = ...,
    include_input: dict[str, Any] | None = ...,
    include_output: dict[str, Any] | None = ...,
    exclude_input: Sequence[str] | None = ...,
    exclude_output: Sequence[str] | None = ...,
    document_output: bool = ...,
    include_signature: bool = ...,
) -> None: ...



"""
Type annotations for botocore.retries.base module.

Copyright 2025 Vlad Emelianov
"""

from botocore.retries.standard import RetryContext

class BaseRetryBackoff:
    def delay_amount(self, context: RetryContext) -> float: ...

class BaseRetryableChecker:
    def is_retryable(self, context: RetryContext) -> bool: ...



"""
Type annotations for botocore.paginate module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Iterator
from logging import Logger
from typing import Any, Generic, TypeVar

from botocore.exceptions import PaginationError as PaginationError
from botocore.model import OperationModel
from botocore.utils import merge_dicts as merge_dicts
from botocore.utils import set_value_from_jmespath as set_value_from_jmespath
from jmespath.parser import ParsedResult

_R = TypeVar("_R")
log: Logger = ...

class TokenEncoder:
    def encode(self, token: dict[str, Any]) -> str: ...

class TokenDecoder:
    def decode(self, token: str) -> dict[str, Any]: ...

class PaginatorModel:
    def __init__(self, paginator_config: Any) -> None: ...
    def get_paginator(self, operation_name: str) -> Any: ...

class PageIterator(Generic[_R]):
    def __init__(
        self,
        method: Any,
        input_token: Any,
        output_token: Any,
        more_results: Any,
        result_keys: Any,
        non_aggregate_keys: Any,
        limit_key: str,
        max_items: int,
        starting_token: Any,
        page_size: int,
        op_kwargs: dict[str, Any],
    ) -> None: ...
    @property
    def result_keys(self) -> list[ParsedResult] | None: ...
    @property
    def resume_token(self) -> dict[str, Any] | None: ...
    @resume_token.setter
    def resume_token(self, value: dict[str, Any] | None) -> None: ...
    @property
    def non_aggregate_part(self) -> dict[str, Any]: ...
    def __iter__(self) -> Iterator[_R]: ...
    def search(self, expression: str) -> Iterator[Any]: ...
    def result_key_iters(self) -> list[ResultKeyIterator[_R]]: ...
    def build_full_result(self) -> dict[str, Any]: ...

class Paginator(Generic[_R]):
    PAGE_ITERATOR_CLS: type[PageIterator[Any]] = ...
    def __init__(
        self,
        method: str,
        pagination_config: dict[str, Any],
        model: OperationModel,
    ) -> None: ...
    @property
    def result_keys(self) -> list[ParsedResult] | None: ...
    def paginate(self, **kwargs: Any) -> PageIterator[_R]: ...

class ResultKeyIterator(Generic[_R]):
    def __init__(self, pages_iterator: PageIterator[_R], result_key: ParsedResult) -> None:
        self.result_key: ParsedResult = ...

    def __iter__(self) -> Iterator[_R]: ...



"""
Type annotations for botocore.httpchecksum module.

Copyright 2025 Vlad Emelianov
"""

import logging
from collections.abc import Iterator, Mapping, Sequence
from typing import IO, Any

from botocore.awsrequest import AWSHTTPResponse
from botocore.compat import HAS_CRT as HAS_CRT
from botocore.exceptions import AwsChunkedWrapperError as AwsChunkedWrapperError
from botocore.exceptions import FlexibleChecksumError as FlexibleChecksumError
from botocore.model import OperationModel
from botocore.response import StreamingBody as StreamingBody
from botocore.utils import determine_content_length as determine_content_length

logger: logging.Logger = ...
DEFAULT_CHECKSUM_ALGORITHM: str = ...

class BaseChecksum:
    def update(self, chunk: bytes | bytearray) -> None: ...
    def digest(self) -> bytes: ...
    def b64digest(self) -> str: ...
    def handle(self, body: bytes | bytearray | IO[Any]) -> str: ...

class Crc32Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class CrtCrc32Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class CrtCrc32cChecksum(BaseChecksum):
    def __init__(self) -> None: ...

class CrtCrc64NvmeChecksum(BaseChecksum):
    def __init__(self) -> None: ...

class Sha1Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class Sha256Checksum(BaseChecksum):
    def __init__(self) -> None: ...

class AwsChunkedWrapper:
    def __init__(
        self,
        raw: IO[Any],
        checksum_cls: type[BaseChecksum] | None = ...,
        checksum_name: str = ...,
        chunk_size: int | None = ...,
    ) -> None: ...
    def seek(self, offset: int, whence: int = ...) -> None: ...
    def read(self, size: int | None = ...) -> bytes: ...
    def __iter__(self) -> Iterator[bytes]: ...

class StreamingChecksumBody(StreamingBody):
    def __init__(
        self, raw_stream: IO[Any], content_length: int, checksum: BaseChecksum, expected: str
    ) -> None: ...
    def read(self, amt: int | None = ...) -> bytes: ...
    def readinto(self, b: IO[bytes]) -> int: ...

def resolve_checksum_context(
    request: Mapping[str, Any], operation_model: OperationModel, params: Mapping[str, Any]
) -> None: ...
def resolve_request_checksum_algorithm(
    request: Mapping[str, Any],
    operation_model: OperationModel,
    params: Mapping[str, Any],
    supported_algorithms: Sequence[str] | None = ...,
) -> None: ...
def apply_request_checksum(request: Mapping[str, Any]) -> None: ...
def resolve_response_checksum_algorithms(
    request: Mapping[str, Any],
    operation_model: OperationModel,
    params: Mapping[str, Any],
    supported_algorithms: Sequence[str] | None = ...,
) -> None: ...
def handle_checksum_body(
    http_response: AWSHTTPResponse,
    response: Mapping[str, Any],
    context: Mapping[str, Any],
    operation_model: OperationModel,
) -> None: ...



"""
Type annotations for botocore.endpoint module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping, Sequence
from logging import Logger
from typing import Any

from botocore.awsrequest import AWSPreparedRequest
from botocore.awsrequest import create_request_object as create_request_object
from botocore.exceptions import HTTPClientError as HTTPClientError
from botocore.history import get_global_history_recorder as get_global_history_recorder
from botocore.hooks import BaseEventHooks
from botocore.hooks import first_non_none_response as first_non_none_response
from botocore.httpsession import URLLib3Session as URLLib3Session
from botocore.model import OperationModel, ServiceModel
from botocore.parsers import ResponseParserFactory
from botocore.response import StreamingBody as StreamingBody
from botocore.utils import get_environ_proxies as get_environ_proxies
from botocore.utils import is_valid_endpoint_url as is_valid_endpoint_url

logger: Logger = ...

history_recorder: Any
DEFAULT_TIMEOUT: int
MAX_POOL_CONNECTIONS: int

def convert_to_response_dict(http_response: Any, operation_model: OperationModel) -> Any: ...

class Endpoint:
    def __init__(
        self,
        host: str,
        endpoint_prefix: str,
        event_emitter: BaseEventHooks,
        response_parser_factory: Any | None = ...,
        http_session: URLLib3Session | None = ...,
    ) -> None:
        self.host: str
        self.http_session: URLLib3Session

    def close(self) -> None: ...
    def make_request(
        self, operation_model: OperationModel, request_dict: Mapping[str, Any]
    ) -> Any: ...
    def create_request(
        self, params: Mapping[str, Any], operation_model: OperationModel | None = ...
    ) -> Any: ...
    def prepare_request(self, request: Any) -> AWSPreparedRequest: ...

class EndpointCreator:
    def __init__(self, event_emitter: BaseEventHooks) -> None: ...
    def create_endpoint(
        self,
        service_model: ServiceModel,
        region_name: str,
        endpoint_url: str,
        verify: bool | None = ...,
        response_parser_factory: ResponseParserFactory | None = ...,
        timeout: float = ...,
        max_pool_connections: int = ...,
        http_session_cls: type[URLLib3Session] = ...,
        proxies: Mapping[str, Any] | None = ...,
        socket_options: Sequence[tuple[int, ...]] | None = ...,
        client_cert: tuple[str, str] | str | None = ...,
        proxies_config: Mapping[str, Any] | None = ...,
    ) -> Any: ...



"""
Type annotations for botocore.docs.sharedexample module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Iterable, Mapping
from typing import Any

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.model import OperationModel, Shape

class SharedExampleDocumenter:
    def document_shared_example(
        self, example: Any, prefix: str, section: DocumentStructure, operation_model: OperationModel
    ) -> None: ...
    def document_input(
        self, section: DocumentStructure, example: Mapping[str, Any], prefix: str, shape: Shape
    ) -> None: ...
    def document_output(
        self, section: DocumentStructure, example: Mapping[str, Any], shape: Shape
    ) -> None: ...

def document_shared_examples(
    section: DocumentStructure,
    operation_model: OperationModel,
    example_prefix: str,
    shared_examples: Iterable[Mapping[str, Any]],
) -> None: ...
